/*
 *  config.c
 *  mod_musicindex
 *
 *  $Id: config.c,v 1.32 2003/09/20 15:15:25 varenet Exp $
 *
 *  Created by Thibaut VARENE on Thu Mar 20 2003.
 *  Copyright (c) 2003 Regis BOUDIN
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2.1, or (at your option)
 *  any later version.
 *
 */

/**
 * @file 
 * Configuration handling and default settings.
 *
 * @author Regis Boudin
 * @version $Revision: 1.32 $
 * @date 2003
 *
 * That file takes care of the module configuration, either via setting some
 * known default values, and/or by getting them from environmental configuration
 * variables passed to apache in its config file.
 *
 * @todo Regis, please document.
 */

#include "config.h"
#include "inf.h"

const char *default_sort = "APNBTLRFMU";
const char *default_fields = "TBLR";

void *create_musicindex_config(pool * p, char *dummy)
{
	unsigned short i;
	mu_config *new = (mu_config *) ap_pcalloc(p, sizeof(mu_config));

	for (i = 0; i < ARG_NUMBER; i++) {
		new->order[i] = SB_DEFAULT;
		new->fields[i] = '\0';
	}

	strcpy(new->fields, default_fields);
	strcpy(new->order, default_sort);

	new->cd_icon = ap_pstrdup(p, "/musicindex/general.png");
	new->small_cd_icon = ap_pstrdup(p, "/musicindex/directory.png");
	new->sound_icon = ap_pstrdup(p, "/musicindex/sound.png");
	new->fetch_icon = ap_pstrdup(p, "/musicindex/fetch.png");
	new->arrow = ap_pstrdup(p, "/musicindex/right_arrow.gif");
	new->css = ap_pstrdup(p, "/musicindex/musicindex.css");

	new->search = NULL;

	new->play_recursive = ~MI_RECURSIVE;
	new->options = MI_RECURSIVE;

	new->cache_path = NULL;
	
	set_fctptrs(new);

	return (void *) new;
}

void *merge_musicindex_configs(pool * p, void *basev, void *addv)
{

	mu_config *new = (mu_config *) ap_pcalloc(p, sizeof(mu_config));
	mu_config *add = (mu_config *) addv;

	strcpy(new->order, add->order);
	strcpy(new->fields, add->fields);

	new->cd_icon = ap_pstrdup(p, add->cd_icon);
	new->small_cd_icon = ap_pstrdup(p, add->small_cd_icon);
	new->sound_icon = ap_pstrdup(p, add->sound_icon);
	new->fetch_icon = ap_pstrdup(p, add->fetch_icon);
	new->arrow = ap_pstrdup(p, add->arrow);
	new->css = ap_pstrdup(p, add->css);
					
	new->search = ap_pstrdup(p, add->search);
					
	new->play_recursive = add->play_recursive;
	new->options = add->options;
	
	new->cache_path = ap_pstrdup(p, add->cache_path);
	
	set_fctptrs(new);

	return new;
}

void sort_or_fields(cmd_parms * cmd, char *list, const char *optstr)
{

	char *r;
	unsigned short i = 0;

	while (optstr[0] && (i < ARG_NUMBER)) {

		r = ap_getword_conf(cmd->pool, &optstr);
		
		if (!strcasecmp(r, "track"))
			list[i++] = SB_TRACK;
		else if (!strcasecmp(r, "disc"))
			list[i++] = SB_POSN;
		else if (!strcasecmp(r, "length"))
			list[i++] = SB_LENGTH;
		else if (!strcasecmp(r, "bitrate"))
			list[i++] = SB_BITRATE;
		else if (!strcasecmp(r, "artist"))
			list[i++] = SB_ARTIST;
		else if (!strcasecmp(r, "album"))
			list[i++] = SB_ALBUM;
		else if (!strcasecmp(r, "title"))
			list[i++] = SB_TITLE;
		else if (!strcasecmp(r, "filename"))
			list[i++] = SB_FILENAME;
		else if (!strcasecmp(r, "date"))
			list[i++] = SB_DATE;
		else if (!strcasecmp(r, "filetype"))
			list[i++] = SB_FILETYPE;
		else if (!strcasecmp(r, "genre"))
			list[i++] = SB_GENRE;
	}
	if (i == ARG_NUMBER)
		i--;
	list[i] = SB_DEFAULT;
}

const char *sort_order(cmd_parms * cmd, void *d, const char *optstr)
{
	mu_config *d_cfg = (mu_config *) d;
	sort_or_fields(cmd, d_cfg->order, optstr);
	set_fctptrs(d_cfg);
	return NULL;
}

const char *set_fields(cmd_parms * cmd, void *d, const char *optstr)
{
	mu_config *d_cfg = (mu_config *) d;
	sort_or_fields(cmd, d_cfg->fields, optstr);
	return NULL;
}

const char *music_lister(cmd_parms * cmd, void *d)
{
	mu_config *d_cfg = (mu_config *) d;
	d_cfg->options |= MI_ACTIVE;
	return NULL;
}

const char *allow_stream(cmd_parms * cmd, void *d)
{
	mu_config *cfg = (mu_config *) d;
	cfg->options |= MI_ALLOWSTREAM;
	return NULL;
}

const char *allow_download(cmd_parms * cmd, void *d)
{
	mu_config *cfg = (mu_config *) d;
	cfg->options |= MI_ALLOWDWNLD;
	return NULL;
}

const char *allow_search(cmd_parms * cmd, void *d)
{
	mu_config *cfg = (mu_config *) d;
	cfg->options |= MI_ALLOWSEARCH;
	return NULL;
}

const char *set_cache_path(cmd_parms * cmd, void *d, const char *optstr)
{
	mu_config *cfg = (mu_config *) d;
	cfg->cache_path = ap_pstrdup(cmd->pool, optstr);
	return NULL;
}
